<?php

namespace App;

use App\JobType;
use Carbon\Carbon;
use App\WorkExperience;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;
use Cviebrock\EloquentSluggable\Sluggable;
use Illuminate\Support\Facades\DB;

class Job extends Model
{
     use Sluggable;

    protected $dates = ['end_date', 'start_date'];

    protected $casts = [
        'required_columns' => 'array',
        'meta_details' => 'array',
        'section_visibility' => 'array'
    ];

    protected $appends = [
        'active'
    ];

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('company', function (Builder $builder) {
            if (auth()->check() && !auth()->user()->is_superadmin) {
                $builder->where('jobs.company_id', user()->company_id);
            }
        });
    }

    public function applications()
    {
        return $this->belongsToMany(JobApplication::class);
    }

    public function category()
    {
        return $this->belongsTo(JobCategory::class, 'category_id');
    }

    public function saasFrontCategory()
    {
        return $this->belongsTo(JobCategory::class, 'category_id')->withoutGlobalScope('company');
    }
    
    public function location()
    {
        return $this->belongsTo(JobLocation::class, 'location_id');
    }

    public function skills()
    {
        return $this->hasMany(JobSkill::class, 'job_id');
    }

    public function company()
    {
        return $this->belongsTo(Company::class, 'company_id');
    }

    public function jobCompany()
    {
        return $this->belongsTo(JobCompany::class, 'job_company_id');
    }

    public function sluggable(): array
    {
        return [
            'slug' => [
                'source' => ['title', 'location.location']
            ]
        ];
    }

    public static function activeJobs()
    {
        return Job::where('status', 'active')
            ->with(['jobLocation'])
            ->where('start_date', '<=', Carbon::now()->format('Y-m-d'))
            ->orWhere('end_date', '>=', Carbon::now()->format('Y-m-d'))
            ->get();
    }

    public static function frontActiveJobs($companyId)
    {
        return Job::withoutGlobalScope('company')
            ->where('status', 'active')
            ->where('start_date', '<=', Carbon::now()->format('Y-m-d'))
            ->where(function($query){
                $query->where('end_date', '>=', Carbon::now()->format('Y-m-d'));
                $query->orWhereNull('end_date');
            })
            ->where('company_id', $companyId)
            ->get();
    }

    public static function activeJobsCount()
    {
        return Job::where('status', 'active')
            ->where('start_date', '<=', Carbon::now()->format('Y-m-d'))
            ->where(function($query){
                $query->where('end_date', '>=', Carbon::now()->format('Y-m-d'));
                $query->orWhereNull('end_date');
            })
            ->count();
    }

    public function getActiveAttribute()
    {
        // return $this->status === 'active' && $this->start_date->lessThanOrEqualTo(Carbon::now()) && $this->end_date->greaterThanOrEqualTo(Carbon::now());
    }

    public function questions()
    {
        return $this->belongsToMany(Question::class, 'job_questions');
    }

    Public function groupQuestions()
    {
        return $this->belongsTo(GroupQuestion::class, 'group_id');
    }

   Public function jobCompanySaas()
   {
         return $this->belongsTo(JobCompany::class, 'job_company_id')->withoutGlobalScope('company');
   }

    public function workExperience()
    {
        return $this->belongsTo(WorkExperience::class, 'work_experience_id');
    }

    public function jobType(){
        return $this->belongsTo(JobType::class, 'job_type_id');
    }
    
    Public function jobLocation(){
        
        return $this->belongsToMany(JobLocation::class, 'job_job_locations', 'job_id', 'location_id');
    }
}
    